<?php
namespace Drupal\msacr\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleExtensionList;
use GuzzleHttp\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;


/**
 * Provides route responses for the msacr module.
 */
class ReportController extends ControllerBase {

  /**
   * The module extension list service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The Guzzle http client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;


  /**
   * {@inheritdoc}
   */
  public function __construct(ModuleExtensionList $module_extension_list, Client $client) {
    $this->moduleExtensionList = $module_extension_list;
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('extension.list.module'),
      $container->get('http_client')
    );
  }

  /**
   * The main report page.
   *
   * @return array
   *   Renderable array.
   */
  public function reportPage() {
    $modules = $this->moduleExtensionList->getList();
    $not_covered = [];
    $covered = [];
    $unknown = [];
    foreach ($modules as $project_key => $module) {
      if ($module->status == 1 && $module->origin !== 'core' && $module->info['package'] !== 'Testing') {
        if (!empty($module->info['version'])) {
          $version = $module->info['version'];
          $response = $this->client->get('https://updates.drupal.org/release-history/' . $project_key . '/all');
          $body = $response->getBody();
          $y = $body->read(9999999999);
          $xml = simplexml_load_string($y);
          if (!$xml) {
            dpm('unable to load release xml for ' . $project_key);
            continue;
          }
          // Try to find the release for the used version.
          $found = FALSE;
          if (!empty($xml->releases->release)) {
            foreach ($xml->releases->release as $release) {
              if ($release->version == $version) {
                $found = TRUE;
                if ($release->security == 'Covered by Drupal\'s security advisory policy') {
                  $covered[] = [
                    $project_key,
                    $version,
                    $this->t('Opted-in to security advisory coverage'),
                  ];
                }
                else {
                  $not_covered[] = [
                    $project_key,
                    $version,
                    $release->security,
                  ];
                }
              }
            }
          }
          if (!$found) {
            $unknown[] = [
              $project_key,
              $version,
              $this->t('Unable to find release information'),
            ];
          }
        }
      }
    }
    $header = [
      $this->t('Module name'),
      $this->t('Version'),
      $this->t('Status'),
    ];
    return [
      [
        '#markup' => '<h2>' . $this->t('Not opted-in') . '</h2>',
      ],
      [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $not_covered,
      ],
      [
        '#markup' => '<h2>' . $this->t('Opted-in') . '</h2>',
      ],
      [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $covered,
      ],
      [
      '#markup' => '<h2>' . $this->t('Unknown') . '</h2>',
    ],
      [
        '#type' => 'table',
        '#header' => [
          $this->t('Module name'),
          $this->t('Version'),
          $this->t('Error'),
        ],
        '#rows' => $unknown,
      ],
    ];

  }

}
